<?php
//-------------------------------------------------------------------------
// This file is part o digitizepoints, a plugin for p.mapper.
// It allow to digitize points into a PostgreSQL/PostGIS table.
// See http://www.pmapper.net/
//
// Copyright (C) 2009 Niccolo Rigacci
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as
// published by the Free Software Foundation, either version 3 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// Author:       Niccolo Rigacci <niccolo@rigacci.org>
//-------------------------------------------------------------------------

require_once 'DB.php';
require_once 'include.php';
require_once 'include_conf.php';

// TODO:
// * Now the pkey must be numeric, add proper escaping if needed.
// * Trap errors on fetchRow, when getting lon/lat of existing point.

$distance  = 'st_distance';     // Name for 'SELECT ... AS', must not conflict with other table fields.
$prefix    = '__db';            // Prefix for database fields used into the html form.

print "<html>\n";
print "<head>\n";
print "</head>\n";
print "<body>\n";

$lon = (float)$_REQUEST['lon'];
$lat = (float)$_REQUEST['lat'];

//------------------------------------------------------------------------
// Connect to the database.
//------------------------------------------------------------------------
$db = DB::connect($dsn, true);
if (DB::isError($db)) die ($db->getMessage());


//------------------------------------------------------------------------
// Do insert.
//------------------------------------------------------------------------
if ($_REQUEST['__action'] == 'insert') {
    // Get all the fields from the web form.
    list($columns, $values) = get_columns_and_values($_REQUEST, $prefix, $db);
    // Add the geometry.
    $val = sprintf('ST_SetSRID(ST_MakePoint(%f, %f), %d)', $lon, $lat, $srid_map);
    if ($srid_geom != $srid_map) $val = "ST_Transform($val, $srid_geom)";
    array_push($columns, $the_geom);
    array_push($values,  $val);
    // Make the SQL statement.
    $sql  = 'INSERT INTO ' . $db_table . ' (' . implode(', ', $columns) . ')';
    $sql .= ' VALUES (' . implode(', ', $values) . ')';
    $result = $db->query($sql);
    if (DB::isError($result)) {
        print "<b>Error executing statement:</b> " . my_html($sql) ."<p>\n";
        die ($result->getMessage());
    } else {
        msg_and_close('Insert successful.');
    }
    exit;
}

//------------------------------------------------------------------------
// Do update.
//------------------------------------------------------------------------
if ($_REQUEST['__action'] == 'update') {
    // Get the record ID.
    $id = (int)$_REQUEST['__id'];
    // Get all the fields from the web form.
    list($columns, $values) = get_columns_and_values($_REQUEST, $prefix, $db);
    // Add the geometry.
    $val = sprintf('ST_SetSRID(ST_MakePoint(%f, %f), %d)', $lon, $lat, $srid_map);
    if ($srid_geom != $srid_map) $val = "ST_Transform($val, $srid_geom)";
    array_push($columns, $the_geom);
    array_push($values,  $val);
    // Make the SQL statement.
    $sql  = 'UPDATE ' . $db_table . ' SET (' . implode(', ', $columns) . ')';
    $sql .= ' = (' . implode(', ', $values) . ')';
    $sql .= ' WHERE ' . $pkey . ' = ' . $id;
    $result = $db->query($sql);
    if (DB::isError($result)) {
        print "<b>Error executing statement:</b> " . my_html($sql) ."<p>\n";
        die ($result->getMessage());
    } else {
        msg_and_close('Update successful.');
    }
    exit;
}

//------------------------------------------------------------------------
// Do delete.
//------------------------------------------------------------------------
if ($_REQUEST['__action'] == 'delete') {
    // Get the record ID.
    $id = (int)$_REQUEST['__id'];
    // Make the SQL statement.
    $sql  = 'DELETE FROM ' . $db_table . ' WHERE ' . $pkey . ' = ' . $id;
    $result = $db->query($sql);
    if (DB::isError($result)) {
        print "<b>Error executing statement:</b> " . my_html($sql) ."<p>\n";
        die ($result->getMessage());
    } else {
        msg_and_close('Delete successful.');
    }
    exit;
}

//------------------------------------------------------------------------
// Get points near the clik and get table info.
//------------------------------------------------------------------------
$point = sprintf("ST_PointFromText('POINT(%f %f)', %d)", $lon, $lat, $srid_map);
$geom_ll = $the_geom;

// Function ST_Distance_Sphere() requires EPSG:4326 lon/lat points.
if ($srid_map  != 4326) $point   = "ST_Transform($point, 4326)";
if ($srid_geom != 4326) $geom_ll = "ST_Transform($the_geom, 4326)";

$sql  = 'SELECT *, ST_Distance_Sphere(%s, %s) AS %s';
$sql .= ' FROM %s WHERE ST_Distance_Sphere(%s, %s) < %f';
$sql .= ' ORDER BY %s';
$sql = sprintf($sql, $geom_ll, $point, $distance, $db_table, $geom_ll, $point, $tolerance, $distance);
print '<!-- ' . my_html($sql) . " -->\n";
$result = $db->query($sql);
if (DB::isError($result)) die ($result->getMessage());
$tableinfo = $result->tableInfo();

// If there is a near point, we will do an update.
if (!isset($_REQUEST['addnew']) and $result->numRows() > 0) {
    $record = $result->fetchRow(DB_FETCHMODE_ASSOC);
    $point = $the_geom;
    if ($srid_geom != $srid_map) $point = "ST_Transform($point, $srid_map)";
    $sql = 'SELECT ST_X(%s), ST_Y(%s) FROM %s WHERE %s = %s';
    $sql = sprintf($sql, $point, $point, $db_table, $pkey, $record[$pkey]);
    list($point_lon, $point_lat) = $db->query($sql)->fetchRow(DB_FETCHMODE_ORDERED);
    $new_record = false;
    $action = 'update';
    $id = $record[$pkey];
} else {
    list($point_lon, $point_lat) = array($lon, $lat);
    $new_record = true;
    $action = 'insert';
    $id = '';
}

$db->disconnect();

//------------------------------------------------------------------------
// Display the insert/update form.
//------------------------------------------------------------------------
$heading = ($new_record) ? 'Insert new point' : 'Update point';
print '<h2>' . my_html($heading) . "</h2>\n";
print '<form id="digitizepoints_form" name="inputform" method="post" action="' . $_SERVER['SCRIPT_NAME'] . '">' . "\n";
print '<input type="hidden" name="__action" value="' . $action . "\">\n";
print '<input type="hidden" name="__id"        id="point_id"  value="' . my_html($id) . "\">\n";
print '<input type="hidden" name="__click_lon" id="click_lon" value="' . my_html($lon) . "\">\n";
print '<input type="hidden" name="__click_lat" id="click_lat" value="' . my_html($lat) . "\">\n";

// Display the form for record insert/update.
print "<table>\n";
foreach ($tableinfo as $f) {
    if (in_array($f['name'], $hide_fields)) continue;
    if ($f['name'] == $the_geom) continue;
    if ($f['name'] == $distance) continue;
    $align = numeric_type($f['type']) ? 'right' : 'left';
    $type = numeric_type($f['type']) ? 'n': 'c';
    print '<tr><th>' . my_html($f['name']) . "</th><td align=\"${align}\">";
    if ($new_record) {
        $value = '';
    } else {
        $value = $record[$f['name']];
    }
    $html_name = $prefix . '_' . $type . '_' . $f['name'];
    print sprintf('<input type="text" size="36" name="%s" value="%s">', my_html($html_name), my_html($value));
    print "</td></tr>\n";
}
// Input fields for longitude and latitude.
print '<tr><th>' . my_html('Longitude') . '</th><td align="right">';
print sprintf('<input type="text" size="36" name="lon" value="%s">', my_html($point_lon));
print "</td></tr>\n";
print '<tr><th>' . my_html('Latitude') . '</th><td align="right">';
print sprintf('<input type="text" size="36" name="lat" value="%s">', my_html($point_lat));
print "</td></tr>\n";

$addnew_url = sprintf('?addnew=yes&lon=%f&lat=%f', $lon, $lat);
$delete_url = sprintf('?__action=delete&__id=%d', my_html($id));
$disabled = $new_record ? 'disabled' : '';

?>
<tr><th>&nbsp;</th><td>
<input type="button" value="Save"   onClick="javascript: PM.Plugin.Digitizepoints.pntSave();" />
<input type="button" value="Cancel" onClick="javascript: PM.Plugin.Digitizepoints.closeDlg();" />
<p>
<input type="button" value="Delete point"         onClick="javascript: if (!confirm('Delete point?')) return false; PM.Plugin.Digitizepoints.pntDelete();"<?= $disabled ?>>
<input type="button" value="Do not edit, add new" onClick="javascript: PM.Plugin.Digitizepoints.pntAddNew();"<?= $disabled ?>>
</td>
</table>

</form>

</body>
</html>
